#pragma once

#include "defs.h"

#include <string>

namespace audio
{
    namespace e_audioStreamingType
    {
        enum audioStreamingType
        {
            nonStreaming,
            streaming
        };
    }
    typedef e_audioStreamingType::audioStreamingType AudioStreamingType;

    namespace e_audioLoopType
    {
        enum audioLoopType
        {
            none,
            looping
        };
    }
    typedef e_audioLoopType::audioLoopType AudioLoopType;

    struct SoundDesc
    {
        AudioStreamingType  m_streamingType = e_audioStreamingType::nonStreaming;
        AudioLoopType       m_loopType = e_audioLoopType::none;
        u32                 m_channels = 1;
        u32                 m_samplesPerSecond = 1;
        u32                 m_bitsPerSample = 16;
        float               m_startTime = 0.0f;
        float               m_loopStartTime = 0.0f;
        float               m_loopEndTime = -1.0f;
        Mem                 m_data;
    };

    typedef SoundDesc* Sound;


    struct AudioDevice
    {
        std::string m_name = "";
        void*       m_device = nullptr;
    };

    class AudioManager
    {
    public:
        virtual void    setup() = 0;
        virtual void    shutdown() = 0;
        virtual void    update(float dt) = 0;

        virtual void    getAudioDevices(AudioDevice*& devices, int& numDevices) = 0;
        virtual void    setAudioDevice(const AudioDevice& device) = 0;

        virtual Sound   loadSound(const SoundDesc& desc) = 0;
        virtual void    unloadSound(Sound sound) = 0;
        virtual void    playSound(Sound sound) = 0;
        virtual void    setSoundPaused(Sound sound, bool paused) const = 0;
        virtual void    seekSound(Sound sound, float t) const = 0;
        virtual void    stopSound(Sound sound) = 0;
        virtual float   getSoundCurrentTime(Sound time) = 0;

        static AudioManager* get();
    };


    AudioManager*   createAudioManager();
    void            destroyAudioManager(AudioManager* manager);
}
