#pragma once

#include <stdlib.h>

typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef unsigned long u64;
typedef u64 ID;

#define ARRAY_SIZE(arr) sizeof(arr) / sizeof(arr[0])

#define RING_BUFFER_SIZE 64

template<typename T>
inline T* memNew()
{
    return new T;
}

template<typename T>
inline void memDelete(T* mem)
{
    delete mem;
}

template<typename T>
inline T* memNewArray(size_t count)
{
    return new T[count];
}

template<typename T>
inline void memDeleteArray(T* mem)
{
    delete[] mem;
}

inline void* memAlloc(size_t size)
{
    return malloc(size);
}

inline void* memRealloc(void* src, size_t size)
{
    return realloc(src, size);
}

inline void memFree(void* mem)
{
    free(mem);
}

template<typename T>
inline ID hash(T data)
{
    return (ID)std::hash<T>{}(data);
}

template<typename T>
inline ID hashCombine(const ID& current, T data)
{
    return current ^ (ID)std::hash<T>{}(data);
}

struct Mem
{
    void* m_data = nullptr;
    size_t  m_size = 0;
};

template<typename T>
class RingBuffer
{
public:
    bool get(T& val)
    {
        if (m_get == m_put)
            return false;

        val = m_data[m_get];
        m_get = (m_get + 1) % RING_BUFFER_SIZE;
        return true;
    }

    void put(T val)
    {
        m_data[m_put] = val;
        m_put = (m_put + 1) % RING_BUFFER_SIZE;
    }

protected:
    T m_data[RING_BUFFER_SIZE];
    size_t m_get = 0;
    size_t m_put = 0;
};