#pragma once

#include "gfx_defs.h"

#include <string>
#include <vector>

namespace gfx
{
    struct Codepoints
    {
        char m_first = '\0';
        char m_last = '\0';
    };
    struct FontDesc
    {
        std::string             m_filename = "";
        float                   m_scale = 1.0f;
        std::vector<Codepoints> m_codepoints;
    };

    namespace e_textHAlign
    {
        enum textHAlign
        {
            left,
            right,
            centre
        };
    }
    typedef e_textHAlign::textHAlign TextHAlign;

    namespace e_textVAlign
    {
        enum textVAlign
        {
            top,
            bottom,
            centre
        };
    }
    typedef e_textVAlign::textVAlign TextVAlign;

    class Font
    {
    public:
        virtual bool            load(const FontDesc& desc) = 0;      
        virtual bool            createTexture() = 0;
        virtual void            unload() = 0;

        virtual Geometry        createTextMesh(const std::string& text, TextHAlign hAlign, TextVAlign vAlign) = 0;
        virtual gfx::Texture    getTexture() = 0;
    };

    Font*   createFont();
    void    destroyFont(Font* font);
}
