#pragma once
#include "gfx_defs.h"

namespace gfx
{
    class Context
    {
    public:
        virtual Shader createShader(const ShaderDesc& desc) = 0;
        virtual Buffer createBuffer(const BufferDesc& desc) = 0;
        virtual Texture createTexture(const TextureDesc& desc) = 0;
        virtual Sampler createSampler(const SamplerDesc& desc) = 0;
        virtual RenderPass createRenderPass(const RenderPassDesc& desc) = 0;
        virtual RenderPipeline createRenderPipeline(const RenderPipelineDesc& desc) = 0;
        virtual ComputePipeline createComputePipeline(const ComputePipelineDesc& desc) = 0;
        virtual Framebuffer obtainFramebuffer(const FramebufferDesc& desc) = 0;

        virtual void destroyShader(Shader shader) = 0;
        virtual void destroyBuffer(Buffer buffer) = 0;
        virtual void destroyTexture(Texture texture) = 0;
        virtual void destroySampler(Sampler sampler) = 0;
        virtual void destroyRenderPass(RenderPass pass) = 0;
        virtual void destroyRenderPipeline(RenderPipeline pipeline) = 0;
        virtual void destroyComputePipeline(ComputePipeline pipeline) = 0;

        virtual void begin() = 0;
        virtual void setViewport(const Viewport& vp) = 0;
        virtual void setScissorRect(const Rect& sr) = 0;
        virtual void beginComputeEncoding() = 0;
        virtual void beginRenderPass(RenderPass pass, Framebuffer framebuffer, Clear* clears = nullptr, size_t numClears = 0) = 0;
        virtual void beginNextSubpass() = 0;
        virtual void bindRenderPipeline(RenderPipeline pipeline) = 0;
        virtual void bindComputePipeline(ComputePipeline pipeline) = 0;
        virtual void bindBuffer(Buffer buf, u32 slot, BindFlags flags, size_t offset = 0) = 0;
        virtual void bindTexture(Texture tex, Sampler samp, u32 slot, BindFlags flags) = 0;
        virtual void updateBuffer(Buffer buf, const Mem& data, size_t offset = 0) = 0;
        virtual void updateTexture(Texture tex, const TextureUpdateDesc& desc) = 0;
        virtual void updateConstants(ShaderStageFlags stage, size_t size, size_t offset, void* data) = 0;
        virtual void insertBarrier(PipelineStage srcStage, PipelineStage dstStage, BarrierFlags barrierType) = 0;
        virtual void copyTexture(Texture src, Texture dst, u32 srcLayer = 0, u32 dstLayer = 0) = 0;
        virtual void generateMips(Texture tex) = 0;

        virtual void draw(u32 vertexCount, u32 startVertex = 0) = 0;
        virtual void drawIndexed(u32 indexCount, Buffer indexBuffer, IndexType indexType, u32 indexBufferOffset = 0, u32 vertexOffset = 0) = 0;
        virtual void drawIndexedInstanced(u32 indexCount, u32 instanceCount, Buffer indexBuffer, IndexType indexType, u32 indexBufferOffset = 0, u32 vertexOffset = 0) = 0;
        virtual void drawIndirect(u32 drawCount, Buffer drawBuffer, u32 drawBufferOffset = 0) = 0;
        virtual void drawIndexedIndirect(u32 drawCount, Buffer indexBuffer, IndexType indexType, Buffer drawBuffer, u32 indexBufferOffset = 0, u32 drawBufferOffset = 0) = 0;
        virtual void dispatchCompute(u32 grid[3], u32 numThreads[3]) = 0;
        virtual void dispatchComputeIndirect(Buffer drawBuffer, u32 drawBufferOffset = 0) = 0;

        virtual void        endRenderEncoding() = 0;
        virtual void        endComputeEncoding() = 0;
        virtual void        enqueue() = 0;
        virtual Semaphore   commit(Semaphore* wait = nullptr, size_t count = 0, PipelineStage waitStage = e_pipelineStageType::top) = 0;
        virtual void        present() = 0;
        virtual void        flushCommandBuffers() = 0;
        virtual bool        isSignalled(Semaphore semaphore) = 0;

        virtual TextureDesc getTextureDesc(Texture tex) = 0;
        virtual TextureDesc getBackbufferTextureDesc() = 0;
    };

    void        createMainContext(void* platform, FeatureFlags featureFlags);
    Context*    getMainContext();
    Context*    getAsyncContext();
    void        destroyContext(Context* context);
    std::string getAPI();
}