#pragma once

#define GLM_FORCE_LEFT_HANDED
#include <glm/glm.hpp>

namespace maths
{
    struct AABB
    {
        glm::vec3 m_min = { -1.0f, -1.0f, -1.0f };
        glm::vec3 m_max = { 1.0f, 1.0f, 1.0f };
    };

    struct OBB
    {
        glm::vec3 m_min = { -1.0f, -1.0f, -1.0f };
        glm::vec3 m_max = { 1.0f, 1.0f, 1.0f };
        glm::vec3 m_centre = { 0.0f, 0.0f, 0.0f };
        glm::vec3 m_extents = { 0.0f, 0.0f, 0.0f };
        glm::vec3 m_axes[3];
    };

    struct Ray
    {
        glm::vec3 m_origin = { 0.0f, 0.0f, 0.0f };
        glm::vec3 m_ray = { 0.0f, 0.0f, 0.0f };
        glm::vec3 m_invRay = { 0.0f, 0.0f, 0.0f };
        int m_sign[3];

        void compute()
        {
            m_invRay.x = 1.0f / m_ray.x;
            m_invRay.y = 1.0f / m_ray.y;
            m_invRay.z = 1.0f / m_ray.z;

            m_sign[0] = m_invRay.x < 0.0f;
            m_sign[1] = m_invRay.y < 0.0f;
            m_sign[2] = m_invRay.z < 0.0f;
        }
    };

    struct Frustum
    {
        float m_nearRight = 0.0f;
        float m_nearTop = 0.0f;
        float m_nearPlane = 0.0f;
        float m_farPlane = 0.0f;
    };


    bool        rayAABBIntersection(const Ray& ray, const AABB& aabb, float& t);
    bool        frustumOBBIntersection(const Frustum& frustum, const glm::mat4& view, const AABB& aabb);
    bool        frustumOBBIntersectionSimple(const glm::mat4& vp, const OBB& obb);
    bool        sphereOBBIntersection(const glm::vec3& centre, float radius, const OBB& obb);
    glm::mat4   createVectorAlignMatrix(const glm::vec3& a, const glm::vec3& b);
    glm::mat4   createLookAtMatrix(const glm::vec3& eye, const glm::vec3& centre, const glm::vec3& up);
    glm::mat4   createRotationMatrix(float yaw, float pitch, float roll);
    Frustum     createFrustum(float near, float far, float fov, float aspect);

    float       generateFbmNoise(glm::vec2 st, float seed);
}