#pragma once
#include "defs.h"
#include <string>
#include <assert.h>

typedef void* (*ThreadFunc)(void*);

typedef void* Thread;

#define ASSERT(cond) assert(cond)

namespace utils
{
    namespace e_threadFlags
    {
        enum threadFlags
        {
            none = 0,
            detached = 1 << 0,
        };
    }
    typedef e_threadFlags::threadFlags ThreadFlags;

    namespace e_keyState
    {
        enum keyState
        {
            up,
            down,
            pressed,
            count
        };
    }
    typedef e_keyState::keyState KeyState;

    namespace e_mouseButton
    {
        enum mouseButton
        {
            left,
            middle,
            right,
            count
        };
    }
    typedef e_mouseButton::mouseButton MouseButton;

    namespace e_mouseButtonState
    {
        enum mouseButtonState
        {
            up,
            down,
            pressed,
            clicked,
            doubleClicked,
            count
        };
    }
    typedef e_mouseButtonState::mouseButtonState MouseButtonState;

    namespace e_key
    {
        enum key
        {
            esc,
            ret,
            tab,
            shift,
            ctrl,
            alt,
            del,
            space,
            up,down,left,right,
            a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z,
            zero,one,two,three,four,five,six,seven,eight,nine,

            count
        };
    }
    typedef e_key::key Key;
    
    namespace e_mouseEventType
    {
        enum mouseEventType
        {
            button,
            move
        };
    }
    typedef e_mouseEventType::mouseEventType MouseEventType;

    namespace e_inputEventType
    {
        enum inputEventType
        {
            mouse,
            key
        };
    }
    typedef e_inputEventType::inputEventType InputEventType;

    struct KeyEvent
    {
        KeyState    m_state = e_keyState::up;
        Key         m_key = e_key::esc;
    };

    struct MousePosEvent
    {
        int             m_posX = 0;
        int             m_posY = 0;
        int             m_prevPosX = 0;
        int             m_prevPosY = 0;
    };

    struct MouseButtonEvent
    {
        MouseButton      m_button = e_mouseButton::left;
        MouseButtonState m_buttonState = e_mouseButtonState::up;
    };

    struct MouseEvent
    {
        MouseEventType  m_type = e_mouseEventType::button;
        union
        {
            MousePosEvent       m_pos;
            MouseButtonEvent    m_button;
        };
    };

    struct InputEvent
    {
        InputEventType  m_type = e_inputEventType::mouse;
        union
        {
            KeyEvent        m_key;
            MouseEvent      m_mouse;
        };
    };

    u32     getWindowWidth();
    u32     getWindowHeight();
    size_t  getFrame();

    bool    fileExists(const char* path);
    void    readFileToBuffer(const char* path, void*& buffer, size_t& size);
    void    writeFileFromBuffer(const char* path, void* buffer, size_t size);
    void    freeFileBuffer(void* buf);
    void    createDirectory(const char* path);

    int     runCommand(const char* command);

    void    setCursorVisible(bool visible);
    void    setWindowFullscreen(bool fullscreen);
    bool    isMouseButtonPressed(MouseButton button);
    bool    isMouseButtonDown(MouseButton button);
    void    getMousePos(int& x, int& y);

    bool    isKeyPressed(Key key);
    bool    isKeyDown(Key key);

    Thread  createThread(ThreadFunc func, void* data, ThreadFlags flags);
    void    destroyThread(Thread thread);

    u64  getSystemTimeMs();
    void sleepMs(u64 ms);
}
